import java.util.*;
import java.io.*;

public class BustleIIConverter
{
  private static final int noRows = 2000;;
  private static final int noColumns = 2000;
  
  private BufferedReader in;
  private PrintWriter out;
  private String line;
  private String rule;
  private int noStates;
  private int noCounts;
  private int pattern[][];
  private int rows;
  private int columns;

  public static void main(String[] args)
  {
    try
    {
      File pathName = new File(args[0]);
      String[] fileNames = pathName.list();
      BustleIIConverter converter = new BustleIIConverter();
      for (int i = 0; i < fileNames.length; i++)
      {  
        File in = new File(pathName.getPath(), fileNames[i]);
        if (in.isDirectory())
          main(new String[] {in.getPath(), args[1]});
        else if (in.getCanonicalPath().endsWith("." + args[1]))
        {
          String s = in.getCanonicalPath();
          System.out.println(s);
          converter.setBufferedReader(new BufferedReader(new FileReader(in)));
          File out = new File(s.substring(0, s.indexOf("." + args[1])) + ".sqc");
          if (! out.exists())
            out.createNewFile();
          converter.setPrintWriter(new PrintWriter(new BufferedWriter(new FileWriter(out))));
          converter.writeRule();
          converter.clearPattern();
          converter.readPattern();
          converter.trimPattern();
          converter.writePattern();
          in.delete();
        }
      }
    }
    catch(IOException e)
    {  
      e.printStackTrace();
    }
  }
  
  public BustleIIConverter()
  {
    pattern = new int[noRows][noColumns];
  }
 
  public void setBufferedReader(BufferedReader br)
  {
    in = br;
  }

  public void setPrintWriter(PrintWriter pw)
  {
    out = pw;
  }

  public String readLine()
  {
    try
    {
      String s = in.readLine();
      if (s == null)
        return null;
      else
        return s.toUpperCase();
    }
    catch(IOException e)
    {  
      e.printStackTrace();
    }
    return null;
  }

  public void writeRule()
  {
    out.println("#Rule = Weighted Generations");
    out.println("#States = 5");
    out.println("#Counts = 9");
    out.println("#SW 0,1,0,0,0");
    out.println("#NW 0,0,0,0,0");
    out.println("#NW 0,2,1,2,0");
    out.println("#NW 0,1,0,1,0");
    out.println("#NW 0,2,1,2,0");
    out.println("#NW 0,0,0,0,0");
    out.println("#RT 0,0,0,1,0,0,0,0,0");
    out.println("#RT 2,2,1,2,1,1,2,1,2");
    out.println("#RT 3,3,3,3,3,3,3,3,3");
    out.println("#RT 4,4,4,4,4,4,4,4,4");
    out.println("#RT 0,0,0,0,0,0,0,0,0");
  }

  public void readPattern()
  {
    int r, c;
    int n;
    char ch;
    String s = "";
    line = readLine();
    while (line != null)
    {  
      if (line.startsWith("#L "))
        s = s + line.substring(3);
      line = readLine();
    }
    r = 0;
    c = 0;
    while (s.length() > 0)
    {
      n = 0;
      ch = s.charAt(0);
      while ((ch >= '0') && (ch <= '9'))
      {
        n = n * 10 + (ch - '0');
        s = s.substring(1);
        ch = s.charAt(0);
      }
      if (n == 0)
        n = 1;
      if (ch == '$')
      {
        r += n;
        c = 0;
      }
      else
        if (ch == '.')
          c += n;
        else
        {
          for (int i = 0;  i < n; i++)
          {
            pattern[r][c] = (byte)(ch - '@');
            c++;
          }
        }
      s = s.substring(1);
    }
  }
 
  public void clearPattern()
  {
    for (int r = 0; r < noRows; r++)
      for (int c = 0; c < noColumns; c++)
        pattern[r][c] = 0;
  }
 
  public void trimPattern()
  {
    rows = 0;
    columns = 0;
    for (int r = 0; r < noRows; r++)
      for (int c = 0; c < noColumns; c++)
        if (pattern[r][c] > 0)
        {
          if (r > rows)
            rows = r;
          if (c > columns)
            columns = c;
        }
    out.println("#Rows = " + ++rows);
    out.println("#Columns = " + ++columns);
  }

  public void writePattern()
  {
    int r, c;
    int ce = columns - 1;
    char state;
    char lastState = '0';
    int n = 0;
    boolean blankLine;
    String s = "";
    for (r = 0; r < rows; r++)
    {
      blankLine = true;
      for (c = 0; c < columns; c++)
      {
        if (pattern[r][c] > 0)
        {
          blankLine = false;
          break;
        }
      }
      if (blankLine)
      {
        if (lastState == '$')
          n++;
        else
        {
          lastState = '$';
          n = 1;
        }
        continue;
      }
      else
        for (c = columns - 1; c >= 0; c--)
        {
          if (pattern[r][c] > 0)
          {
            ce = c;
            break;
          }
        }
      for (c = 0; c <= ce;  c++)
      {
        state = (char)(pattern[r][c] + 64);
        if (state == '@')
          state = '.';
        if (n == 0)
        {
          lastState = state;
          n = 1;
        }
        else if (state == lastState)
          n++;
        else
        {
          if (n > 1)
            s = s + n;
          s = s + lastState;
          if (s.length() >= 67)
          {
            out.println("#L " + s);
            s = "";
          }
          lastState = state;
          n = 1;
        }
      }
      if (lastState == '$')
        n++;
      else
      {
        if (n > 1)
          s = s + n;
        s = s + lastState;
        if (s.length() >= 67)
        {
          out.println("#L " + s);
          s = "";
        }
        lastState = '$';
        n = 1;
      }
    }
    if (s != "")
      out.println("#L " + s);
    out.close();
  }
}
