import java.io.*;

public class LifToSqc
{
  private static final int neighborhood[][] = 
  {
    {   0,   0,   0,   0 },

    {   1,   4,  16,  64 },
    {   2,   8,  32, 128 },

    {   3,  12,  48, 192 },
    {   5,  20,  80,  65 },
    {   6,  24,  96, 129 },
    {   9,  36, 144,  66 },
    {  10,  40, 160, 130 },
    {  17,  68,  17,  68 },
    {  18,  72,  33, 132 },
    {  34, 136,  34, 136 },

    {   7,  28, 112, 193 },
    {  11,  44, 176, 194 },
    {  13,  52, 208,  67 },
    {  14,  56, 224, 131 },
    {  19,  76,  49, 196 },
    {  21,  84,  81,  69 },
    {  22,  88,  97, 133 },
    {  25, 100, 145,  70 },
    {  26, 104, 161, 134 },
    {  35, 140,  50, 200 },
    {  37, 148,  82,  73 },
    {  38, 152,  98, 137 },
    {  41, 164, 146,  74 },
    {  42, 168, 162, 138 },

    {  15,  60, 240, 195 },
    {  23,  92, 113, 197 },
    {  27, 108, 177, 198 },
    {  29, 116, 209,  71 },
    {  30, 120, 225, 135 },
    {  39, 156, 114, 201 },
    {  43, 172, 178, 202 },
    {  45, 180, 210,  75 },
    {  46, 184, 226, 139 },
    {  51, 204,  51, 204 },
    {  53, 212,  83,  77 },
    {  54, 216,  99, 141 },
    {  57, 228, 147,  78 },
    {  58, 232, 163, 142 },
    {  85,  85,  85,  85 },
    {  86,  89, 101, 149 },
    {  90, 105, 165, 150 },
    { 102, 153, 102, 153 },
    { 106, 169, 166, 154 },
    { 170, 170, 170, 170 },

    {  31, 124, 241, 199 },
    {  47, 188, 242, 203 },
    {  55, 220, 115, 205 },
    {  59, 236, 179, 206 },
    {  61, 244, 211,  79 },
    {  62, 248, 227, 143 },
    {  87,  93, 117, 213 },
    {  91, 109, 181, 214 },
    {  94, 121, 229, 151 },
    { 103, 157, 118, 217 },
    { 107, 173, 182, 218 },
    { 110, 185, 230, 155 },
    { 122, 233, 167, 158 },
    { 171, 174, 186, 234 },

    {  63, 252, 243, 207 },
    {  95, 125, 245, 215 },
    { 111, 189, 246, 219 },
    { 119, 221, 119, 221 },
    { 123, 237, 183, 222 },
    { 126, 249, 231, 159 },
    { 175, 190, 250, 235 },
    { 187, 238, 187, 238 },

    { 127, 253, 247, 223 },
    { 191, 254, 251, 239 },

    { 255, 255, 255, 255 }
  };

  private static final int fullSymmetry[][] =
  {
    { 3,  5},
    { 6,  9},
    {12, 19},
    {13, 17},
    {15, 18},
    {20, 22},
    {25, 29},
    {26, 28},
    {31, 43},
    {32, 41},
    {33, 38},
    {34, 42},
    {35, 40},
    {46, 57},
    {47, 54},
    {48, 56},
    {49, 53},
    {59, 64},
    {61, 63}
  };

  private static final int lifPatternSize = 50;

  private String fileName;
  private String rule;
  private int birthNeighbors[];
  private int surviveNeighbors[];
  private int ruleTable[];
  private String[] lifPattern;
  private int pattern[][];
  private int rows;
  private int columns;

  public LifToSqc(String fileName, File in, File out)
  {
    this.fileName = fileName;
    try
    {
      BufferedReader br = new BufferedReader(new FileReader(in));
      readFile(br);
      br.close();
      PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(out)));
      writeFile(pw);
      pw.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  private void readFile(BufferedReader br)
  {
    try
    {
      rule = br.readLine().substring(3);
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    String s = rule;
    if (!s.substring(0, 1).equals("s"))
    {
      System.out.println("Rule String Error: " + fileName);
      System.exit(1);
    }
    int i = s.indexOf("/");
    String survive = s.substring(1, i);
    s = s.substring(i + 1);
    if (!s.substring(0, 1).equals("b"))
    {
      System.out.println("Rule String Error: " + fileName);
      System.exit(1);
    }
    String birth = s.substring(1);
    birthNeighbors = new int[70];
    makeRuleFromString(birth, birthNeighbors);
    surviveNeighbors = new int[70];
    makeRuleFromString(survive, surviveNeighbors);
    makeRuleTable();
    lifPattern = new String[lifPatternSize];
    try
    {
      rows = 0;
      while ((s = br.readLine()) != null)
      {
        if (!s.startsWith("#P"))
          if (rows < lifPatternSize)
          {
            lifPattern[rows] = s;
            rows++;
          }
          else
          {
            System.out.println("Pattern Error: " + fileName);
            System.exit(1);
          }
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    makePattern();
  }

  private void makeRuleFromString(String s, int[] n)
  {
    String stringOffsets[] =
    {
      "0", "ce", "acakevki", "ijvaqcvqjyryke", "avwvatjkyzriqycrkzje", "avyqjiekjyrqvc", "aekikacv", "ec", "8"
    };
    String so = "";
    int offsets[] = {0, 1, 3, 11, 25, 45, 59, 67, 69};
    int index;
    int offset = 0;
    
    if (s.equals(""))
      return;
    for (int i = 0; i < 70; i++)
      n[i] = 0;
    while (s.length() > 0)
    {
      String t = s.substring(0, 1);
      s = s.substring(1);
      if (t.equals("0"))
        n[0] = 1;
      else if (t.equals("8"))
        n[69] = 1;
      else if ((t.compareTo("0") > 0) && (t.compareTo("8") < 0))
      {
        index = Integer.parseInt(t);
        offset = offsets[index];
        so = stringOffsets[index];
        if ((s.length() == 0) || ((s.substring(0, 1).compareTo("0") >= 0) && (s.substring(0, 1).compareTo("8") <= 0)))
        {
          for (int i = 0; i < so.length(); i++)
            n[offset + i] = 1;
        }
      }
      else
        for (int i = 0; i < so.length(); i++)
          if (t.equals(so.substring(i, i + 1)))
            n[offset + i] = 1;
    }
  }

  private void makeRuleTable()
  {
    ruleTable = new int[256];
    for (int i = 0; i < 256; i++)
      ruleTable[i] = 0;
    for (int i = 0; i < 70; i++)
    {
      int birth = 0;
      if (birthNeighbors[i] == 1)
        birth = 2;
      int survive = 0;
      if (surviveNeighbors[i] == 1)
        survive = 1;
      for (int j = 0; j < 4; j++)
        ruleTable[neighborhood[i][j]] = birth + survive;
    }
  }

  private void makePattern()
  {
    columns = lifPattern[0].length();
    for (int i = 1; i < rows; i++)
      if (lifPattern[i].length() != columns)
      {
        System.out.println("Pattern Error: " + fileName);
        System.exit(1);
      }
    pattern = new int[rows][columns];
    for (int r = 0; r < rows; r++)
      for (int c = 0; c < columns; c++)
        pattern[r][c] = 0;
    for (int r = 0; r < rows; r++)
      for (int c = 0; c < columns; c++)
        if (lifPattern[r].substring(c, c + 1).equals("*"))
          pattern[r][c] = 1;
  }

  private void writeFile(PrintWriter pw)
  {
    writeRule(pw);
    pw.println("#Rows = " + rows);
    pw.println("#Columns = " + columns);
    writePattern(pw);    
  }

  private void writeRule(PrintWriter pw)
  {
    pw.println("#Rule = Non Totalistic");
    String s = "#RT ";
    for (int i = 0; i < 256; i++)
    {
      if (((i % 64) == 0) && (i > 0))
      {
        pw.println(s);
        s = "#RT ";
      }
      s = s + ruleTable[i];
    }
    pw.println(s);
  }

  private void writePattern(PrintWriter pw)
  {
    int r, c;
    int ce = columns - 1;
    char state;
    char lastState = '0';
    int n = 0;
    boolean blankLine;
    String s = "";
    for (r = 0; r < rows; r++)
    {
      blankLine = true;
      for (c = 0; c < columns; c++)
      {
        if (pattern[r][c] > 0)
        {
          blankLine = false;
          break;
        }
      }
      if (blankLine)
      {
        if (lastState == '$')
          n++;
        else
        {
          lastState = '$';
          n = 1;
        }
        continue;
      }
      else
        for (c = columns - 1; c >= 0; c--)
        {
          if (pattern[r][c] > 0)
          {
            ce = c;
            break;
          }
        }
      for (c = 0; c <= ce;  c++)
      {
        state = (char)(pattern[r][c] + 64);
        if (state == '@')
          state = '.';
        if (n == 0)
        {
          lastState = state;
          n = 1;
        }
        else if (state == lastState)
          n++;
        else
        {
          if (n > 1)
            s = s + n;
          s = s + lastState;
          if (s.length() >= 67)
          {
            pw.println("#L " + s);
            s = "";
          }
          lastState = state;
          n = 1;
        }
      }
      if (lastState == '$')
        n++;
      else
      {
        if (n > 1)
          s = s + n;
        s = s + lastState;
        if (s.length() >= 67)
        {
          pw.println("#L " + s);
          s = "";
        }
        lastState = '$';
        n = 1;
      }
    }
    if (s != "")
      pw.println("#L " + s);
  }

  public static void main(String[] args)
  {
    System.out.println(args[0]);
    String inFileName = args[0] + ".lif";
    String outFileName = args[0] + ".sqc";
    File fileIn = new File(inFileName);
    File fileOut = new File(outFileName);
    LifToSqc lifToSqc = new LifToSqc(args[0], fileIn, fileOut);
  }
} 