import java.util.*;
import java.io.*;

public class OldToNewConverter
{
  private BufferedReader in;
  private PrintWriter out;
  private String line;
  private String rule;
  private int noStates;
  private int noCounts;
  private int stateWeights[];
  private int neighborWeights[][];
  private int ruleTable[][];
  private int pattern[][];
  private int rows;
  private int columns;

  public static void main(String[] args)
  {
    try
    {
      File pathName = new File(args[0]);
      String[] fileNames = pathName.list();
      OldToNewConverter converter = new OldToNewConverter();
      for (int i = 0; i < fileNames.length; i++)
      {  
        File in = new File(pathName.getPath(), fileNames[i]);
        if (in.isDirectory())
          main(new String[] {in.getPath()});
        else if (in.getCanonicalPath().endsWith(".sqc"))
        {
          String s = in.getCanonicalPath();
          System.out.println(s);
          converter.setBufferedReader(new BufferedReader(new FileReader(in)));
          converter.readRule();
          converter.readPattern();
          in.delete();
          File out = new File(pathName.getPath(), fileNames[i]);
          if (! out.exists())
            out.createNewFile();
          converter.setPrintWriter(new PrintWriter(new BufferedWriter(new FileWriter(out))));
          converter.writeHeader();
          converter.writeRule();
          converter.writePattern();
        }
      }
    }
    catch(IOException e)
    {  
      e.printStackTrace();
    }
  }
  
  public OldToNewConverter()
  {
    neighborWeights = new int[5][5];
  }
 
  public void setBufferedReader(BufferedReader br)
  {
    in = br;
  }

  public void setPrintWriter(PrintWriter pw)
  {
    out = pw;
  }

	private void readRule()
	{
		{
			try
			{
				rows = new Integer(in.readLine().substring(8));
				columns = new Integer(in.readLine().substring(11));
        pattern = new int[rows][columns];
				noStates = new Integer(in.readLine().substring(12));
				noCounts = new Integer(in.readLine().substring(12));
				stateWeights = new int[noStates];
				ruleTable = new int[noStates][noCounts];
				String line = in.readLine().substring(1);
				for (int i = 0; i < noStates; i++)
				{
					line = line.substring(3);
					stateWeights[i] = new Integer(line.substring(0, 2));
				}
				for (int i = 0; i < 5; i++)
				{
					line = in.readLine().substring(1);
					for (int j = 0; j < 5; j++)
					{
						line = line.substring(3);
						neighborWeights[i][j] = new Integer(line.substring(0, 2));
					}
				}
				for (int i = 0; i < noStates; i++)
				{
					line = in.readLine().substring(1);
					for (int j = 0; j < noCounts; j++)
					{
						line = line.substring(3);
						ruleTable[i][j] = new Integer(line.substring(0, 2));
					}
				}
			}
			catch (IOException e)
			{
        e.printStackTrace();
			}
		}
	}

  public void writeHeader()
  {
    out.println("#Rule = Rule Table");
  }

  public void writeRule()
  {
    out.println("#States = " + noStates);
    out.println("#Counts = " + noCounts);
    String s = "";
    for (int i = 0; i < stateWeights.length; i++)
      s += stateWeights[i] + ",";
    out.println("#SW " + s.substring(0, s.lastIndexOf(",")));
    for (int i = 0; i < 5; i++)
    {
      s = "";
      for(int j = 0; j < 5; j++)
        s += neighborWeights[i][j] + ",";
      out.println("#NW " + s.substring(0, s.lastIndexOf(",")));
    }
    for (int i = 0; i < noStates; i++)
    {
      s = "";
      for(int j = 0; j < noCounts; j++)
        s += ruleTable[i][j] + ",";
      out.println("#RT " + s.substring(0, s.lastIndexOf(",")));
    }
  }

  private String readLine()
  {
    try
    {
      String s = in.readLine();
      if (s == null)
        return null;
      else
        return s.toUpperCase();
    }
    catch(IOException e)
    {  
      e.printStackTrace();
    }
    return null;
  }

  public void readPattern()
  {
    int r, c;
    int n;
    char ch;
    String s = "";
    line = readLine();
    while (line != null)
    {  
      if (line.startsWith("#L "))
        s = s + line.substring(3);
      line = readLine();
    }
    r = 0;
    c = 0;
    while (s.length() > 0)
    {
      n = 0;
      ch = s.charAt(0);
      while ((ch >= '0') && (ch <= '9'))
      {
        n = n * 10 + (ch - '0');
        s = s.substring(1);
        ch = s.charAt(0);
      }
      if (n == 0)
        n = 1;
      if (ch == '$')
      {
        r += n;
        c = 0;
      }
      else
        if (ch == '.')
          c += n;
        else
        {
          for (int i = 0;  i < n; i++)
          {
            pattern[r][c] = (byte)(ch - '@');
            c++;
          }
        }
      s = s.substring(1);
    }
  }

  public void writePattern()
  {
    int r, c;
    int ce = columns - 1;
    char state;
    char lastState = '0';
    int n = 0;
    boolean blankLine;
    out.println("#Rows = " + rows);
    out.println("#Columns = " + columns);
    String s = "";
    for (r = 0; r < rows; r++)
    {
      blankLine = true;
      for (c = 0; c < columns; c++)
      {
        if (pattern[r][c] > 0)
        {
          blankLine = false;
          break;
        }
      }
      if (blankLine)
      {
        if (lastState == '$')
          n++;
        else
        {
          lastState = '$';
          n = 1;
        }
        continue;
      }
      else
        for (c = columns - 1; c >= 0; c--)
        {
          if (pattern[r][c] > 0)
          {
            ce = c;
            break;
          }
        }
      for (c = 0; c <= ce;  c++)
      {
        state = (char)(pattern[r][c] + 64);
        if (state == '@')
          state = '.';
        if (n == 0)
        {
          lastState = state;
          n = 1;
        }
        else if (state == lastState)
          n++;
        else
        {
          if (n > 1)
            s = s + n;
          s = s + lastState;
          if (s.length() >= 67)
          {
            out.println("#L " + s);
            s = "";
          }
          lastState = state;
          n = 1;
        }
      }
      if (lastState == '$')
        n++;
      else
      {
        if (n > 1)
          s = s + n;
        s = s + lastState;
        if (s.length() >= 67)
        {
          out.println("#L " + s);
          s = "";
        }
        lastState = '$';
        n = 1;
      }
    }
    if (s != "")
      out.println("#L " + s);
    out.close();
  }
}
